@@ -6,6 +6,7 @@ use warnings;
 use base qw( Exporter );
 use Wasm
   -api => 0,
+  -exporter => 'ok',
   -wat => q{
     (module
       (func (export "add") (param i32 i32) (result i32)
@@ -20,6 +21,4 @@ use Wasm
     )
   };
 
-our @EXPORT_OK = qw( add subtract );
-
 1;
