@@ -6,7 +6,7 @@ use 5.008001;
 use Carp ();
 
 # ABSTRACT: Write Perl extensions using Wasm
-our $VERSION = '0.02'; # VERSION
+our $VERSION = '0.03'; # VERSION
 
 
 sub import
@@ -24,6 +24,7 @@ sub import
   my $api;
   my $exporter;
   my @module;
+  my @imports;
   my $package = $caller;
 
   while(@_)
@@ -84,6 +85,10 @@ sub import
     {
       $package = shift;
     }
+    elsif($key eq '-imports')
+    {
+      @imports = @{ shift() };
+    }
     else
     {
       Carp::croak("Unknown Wasm option: $key");
@@ -98,7 +103,7 @@ sub import
   my $engine = Wasm::Wasmtime::Engine->new($config);
   my $store = Wasm::Wasmtime::Store->new($engine);
   my $module = Wasm::Wasmtime::Module->new($store, @module);
-  my $instance = Wasm::Wasmtime::Instance->new($module, []);
+  my $instance = Wasm::Wasmtime::Instance->new($module, \@imports);
 
   my @me = $module->exports;
   my @ie = $instance->exports;
@@ -149,7 +154,7 @@ Wasm - Write Perl extensions using Wasm
 
 =head1 VERSION
 
-version 0.02
+version 0.03
 
 =head1 SYNOPSIS
 
@@ -162,6 +167,7 @@ lib/MathStuff.pm:
  use base qw( Exporter );
  use Wasm
    -api => 0,
+   -exporter => 'ok',
    -wat => q{
      (module
        (func (export "add") (param i32 i32) (result i32)
@@ -176,8 +182,6 @@ lib/MathStuff.pm:
      )
    };
  
- our @EXPORT_OK = qw( add subtract );
- 
  1;
 
 mathstuff.pl:
@@ -208,7 +212,7 @@ As of this writing, since the API is subject to change, this must be provided an
  use Wasm -api => 0, -exporter => 'ok';
 
 Configure the caller as an L<Exporter>, with all the functions in the WebAssembly either C<@EXPORT> (C<all>)
-or C<@EXPORT_OK> C(<ok>).
+or C<@EXPORT_OK> (C<ok>).
 
 =head2 -file
 
@@ -216,6 +220,12 @@ or C<@EXPORT_OK> C(<ok>).
 
 Path to a WebAssembly file in either WebAssembly Text (.wat) or WebAssembly binary (.wasm) format.
 
+=head2 -imports
+
+ use Wasm -api => 0, -imports => \@imports;
+
+Use the given imports when creating the module instance.
+
 =head2 -package
 
  use Wasm -api => 0, -package => $package;
