@@ -9,6 +9,7 @@ use Wasm::Wasmtime::Extern;
 use Wasm::Wasmtime::ExternType;
 use Wasm::Wasmtime::Func;
 use Wasm::Wasmtime::FuncType;
+use Wasm::Wasmtime::ImportType;
 use Wasm::Wasmtime::Instance;
 use Wasm::Wasmtime::Module;
 use Wasm::Wasmtime::Store;
@@ -18,7 +19,7 @@ use Wasm::Wasmtime::WasiConfig;
 use Wasm::Wasmtime::WasiInstance;
 
 # ABSTRACT: Perl interface to Wasmtime
-our $VERSION = '0.02'; # VERSION
+our $VERSION = '0.03'; # VERSION
 
 
 1;
@@ -35,7 +36,7 @@ Wasm::Wasmtime - Perl interface to Wasmtime
 
 =head1 VERSION
 
-version 0.02
+version 0.03
 
 =head1 SYNOPSIS
 
@@ -87,25 +88,24 @@ version 0.02
    )
  });
  
- my $hello = Wasm::Wasmtime::Func->new(
-   $store,
-   Wasm::Wasmtime::FuncType->new([],[]),
-   sub { print "hello world!\n" },
- );
+ sub hello
+ {
+   print "hello world!\n";
+ }
  
- my $instance = Wasm::Wasmtime::Instance->new( $module, [$hello] );
+ my $instance = Wasm::Wasmtime::Instance->new( $module, [\&hello] );
  
  # call a WebAssembly function that calls back into Perl space
- $instance->get_export('call_hello')->();
+ $instance->get_export('call_hello')->as_func;
  
  # call plain WebAssembly function
- my $gcd = $instance->get_export('gcd');
+ my $gcd = $instance->get_export('gcd')->as_func;
  print $gcd->(6,27), "\n";      # 3
  
  # write to memory from Perl and read it from WebAssembly
  my $memory = $instance->get_export('memory')->as_memory;
  poke($memory->data + 10, 42);  # set offset 10 to 42
- my $load = $instance->get_export('load');
+ my $load = $instance->get_export('load')->as_func;
  print $load->(10), "\n";       # 42
  poke($memory->data + 10, 52);  # set offset 10 to 52
  print $load->(10), "\n";       # 52
@@ -154,6 +154,14 @@ WebAssembly System Interface (WASI).
 
 Configuration and access control for WebAssembly System Interface (WASI).
 
+=item L<https://github.com/bytecodealliance/wasmtime>
+
+The rust library used by this module, via its C API, via FFI.
+
+=item L<https://github.com/bytecodealliance/wasmtime-py>
+
+These bindings here heavily influenced by the Python Wasmtime bindings.
+
 =back
 
 =head1 AUTHOR
