@@ -16,7 +16,7 @@ use overload
   ;
 
 # ABSTRACT: Wasmtime function class
-our $VERSION = '0.02'; # VERSION
+our $VERSION = '0.03'; # VERSION
 
 
 $ffi_prefix = 'wasm_func_';
@@ -24,11 +24,11 @@ $ffi->type('opaque' => 'wasm_func_t');
 
 # CBC is probably not how we want to do this long term, but atm
 # Platypus does not support Unions or arrays of records so.
-my $c = Convert::Binary::C->new(
+my $cbc = Convert::Binary::C->new(
   Alignment => 8,
   LongSize => 8, # CBC does not apparently use the native alignment by default *sigh*
 );
-$c->parse(<<'END');
+$cbc->parse(<<'END');
 typedef struct wasm_val_t {
   unsigned char kind;
   union {
@@ -40,7 +40,6 @@ typedef struct wasm_val_t {
     void *funcref;
   } of;
 } wasm_val_t;
-typedef
 typedef wasm_val_t wasm_val_vec_t[];
 END
 
@@ -56,23 +55,44 @@ $ffi->attach( new => ['wasm_store_t', 'wasm_functype_t', '(opaque,opaque)->opaqu
        ? (Wasm::Wasmtime::FuncType->new($_[0], $_[1]), $_[2])
        : @_;
 
-    Carp::croak("FIXME: function with parameters") if scalar $functype->params;
-    Carp::croak("FIXME: function with results")    if scalar $functype->results;
+    my @param_types = map { $_->kind } $functype->params;
+    my $param_string = "record(@{[ $cbc->sizeof('wasm_val_t') * scalar(@param_types) ]})*";
+    my @result_types = map { [ $_->kind, $_->kind_num ] } $functype->results;
+    my $result_string = "string(@{[ $cbc->sizeof('wasm_val_t') * scalar(@result_types) ]})*";
 
     $wrapper = $ffi->closure(sub {
       my($params, $results) = @_;
 
+      my @args = @param_types > 0 ? (do {
+        my @copy = @param_types;
+        map { $_->{of}->{shift @copy} } @{ $cbc->unpack('wasm_val_vec_t', $ffi->cast('opaque' => $param_string, $params)) };
+      }) : ();
+
       local $@ = '';
-      eval {
-        $cb->();
+      my @ret = eval {
+        $cb->(@args);
       };
       if(my $error = $@)
       {
-        my $trap = Wasm::Wasmtime::Trap->new("$error");
+        my $trap = Wasm::Wasmtime::Trap->new($store, "$error\0");
         return delete $trap->{ptr};
       }
       else
       {
+        if(@result_types > 0)
+        {
+          my @ret2 = map {
+            {
+              kind => $_->[1],
+              of => {
+                $_->[0] => shift @ret,
+              }
+            }
+          } @result_types;
+          my $packed = $cbc->pack('wasm_val_vec_t', \@ret2);
+          my $ffi = FFI::Platypus->new( api => 1, lib => [undef] );
+          $ffi->function( memcpy => ['opaque','string','size_t'] => 'opaque' )->call($results, $packed, length($packed));
+        }
         return undef;
       }
     });
@@ -95,7 +115,7 @@ $ffi->attach( call => ['wasm_func_t', 'string', 'string'] => 'wasm_trap_t' => su
   my $xsub = shift;
   my $self = shift;
   my @args = @_;
-  my $args = $c->pack('wasm_val_vec_t', [map {
+  my $args = $cbc->pack('wasm_val_vec_t', [map {
     my $valtype = $_;
     {
       kind => $valtype->kind_num,
@@ -104,7 +124,7 @@ $ffi->attach( call => ['wasm_func_t', 'string', 'string'] => 'wasm_trap_t' => su
       },
     }
   } $self->type->params]);
-  my $results = $c->pack('wasm_val_vec_t', [map { { } } $self->type->results]);
+  my $results = $cbc->pack('wasm_val_vec_t', [map { { } } $self->type->results]);
   my $trap = $xsub->($self->{ptr}, $args, $results);
   if($trap)
   {
@@ -117,7 +137,7 @@ $ffi->attach( call => ['wasm_func_t', 'string', 'string'] => 'wasm_trap_t' => su
   my @results = map {
     my $valtype = shift @valtypes;
     $_->{of}->{$valtype->kind};
-  } @{ $c->unpack('wasm_val_vec_t', $results) };
+  } @{ $cbc->unpack('wasm_val_vec_t', $results) };
   wantarray ? @results : $results[0]; ## no critic (Freenode::Wantarray)
 });
 
@@ -156,6 +176,15 @@ $ffi->attach( result_arity => ['wasm_func_t'] => 'size_t' => sub {
   $xsub->($self->{ptr});
 });
 
+
+# actually returns a wasm_extern_t, but recursion
+$ffi->attach( as_extern => ['wasm_func_t'] => 'opaque' => sub {
+  my($xsub, $self) = @_;
+  require Wasm::Wasmtime::Extern;
+  my $ptr = $xsub->($self->{ptr});
+  Wasm::Wasmtime::Extern->new($ptr, $self->{owner} || $self);
+});
+
 _generate_destroy();
 
 1;
@@ -172,7 +201,7 @@ Wasm::Wasmtime::Func - Wasmtime function class
 
 =head1 VERSION
 
-version 0.02
+version 0.03
 
 =head1 SYNOPSIS
 
@@ -280,6 +309,12 @@ Returns the number of arguments the function takes.
 
 Returns the number of results the function returns.
 
+=head2 as_extern
+
+ my $extern = $func->as_extern;
+
+Returns the L<Wasm::Wasmtime::Extern> for this function.
+
 =head1 SEE ALSO
 
 =over 4
