@@ -5,31 +5,84 @@ use warnings;
 use Wasm::Wasmtime::FFI;
 use Wasm::Wasmtime::Module;
 use Wasm::Wasmtime::Extern;
+use Wasm::Wasmtime::Func;
 use Wasm::Wasmtime::Trap;
+use Ref::Util qw( is_blessed_ref is_plain_coderef is_plain_scalarref );
+use Carp ();
 
 # ABSTRACT: Wasmtime instance class
-our $VERSION = '0.02'; # VERSION
+our $VERSION = '0.03'; # VERSION
 
 
 $ffi_prefix = 'wasm_instance_';
 $ffi->type('opaque' => 'wasm_instance_t');
 
 
-$ffi->attach( new => ['wasm_store_t','wasm_module_t','wasm_extern_t[]','opaque*'] => 'wasm_engine_t' => sub {
+sub _cast_import
+{
+  my($ii, $mi, $store, $keep) = @_;
+  if(ref($ii) eq 'Wasm::Wasmtime::Extern')
+  {
+    return $ii->{ptr};
+  }
+  elsif(is_blessed_ref($ii) && $ii->can('as_extern'))
+  {
+    return $ii->as_extern->{ptr};
+  }
+  elsif(is_plain_coderef($ii))
+  {
+    if($mi->type->kind eq 'func')
+    {
+      my $f = Wasm::Wasmtime::Func->new(
+        $store,
+        $mi->type->as_functype,
+        $ii,
+      );
+      push @$keep, $f;
+      return $f->as_extern->{ptr};
+    }
+  }
+  elsif(is_plain_scalarref($ii) || !defined $ii)
+  {
+    if($mi->type->kind eq 'memory')
+    {
+      my $m = Wasm::Wasmtime::Memory->new(
+        $store,
+        $mi->type->as_memorytype,
+      );
+      $$ii = $m if defined $ii;
+      push @$keep, $m;
+      return $m->as_extern->{ptr};
+    }
+  }
+  Carp::croak("Non-extern object as import");
+}
+
+$ffi->attach( new => ['wasm_store_t','wasm_module_t','wasm_extern_t[]','opaque*'] => 'wasm_instance_t' => sub {
   my($xsub, $class, $module, $imports) = @_;
-  my @imports = defined $imports ? map { $_->{ptr} } @$imports : ();
+  $imports ||= [];
+  my @imports = @$imports;
   my $trap;
+  my $store = $module->store;
+  my @keep;
 
-  # TODO: we don't have an interface to module imports yet, but when
-  # we do we should validate that the module and instance imports match
-  # otherwise SEGV
+  {
+    my @mi = $module->imports;
+    if(@mi != @imports)
+    {
+      Carp::croak("Got @{[ scalar @imports ]} imports, but expected @{[ scalar @mi ]}");
+    }
 
-  my $ptr = $xsub->($module->store->{ptr}, $module->{ptr}, \@imports, \$trap);
+    @imports = map { _cast_import($_, shift @mi, $store, \@keep) } @imports;
+  }
+
+  my $ptr = $xsub->($store->{ptr}, $module->{ptr}, \@imports, \$trap);
   if($ptr)
   {
     return bless {
       ptr    => $ptr,
       module => $module,
+      keep   => \@keep,
     }, $class;
   }
   else
@@ -90,7 +143,7 @@ Wasm::Wasmtime::Instance - Wasmtime instance class
 
 =head1 VERSION
 
-version 0.02
+version 0.03
 
 =head1 SYNOPSIS
 
@@ -118,7 +171,28 @@ This class represents an instance of a WebAssembly module L<Wasm::Wasmtime::Modu
    \@imports,  # array reference of Wasm::Wasmtime::Extern
  );
 
-Create a new instance of the instance class.
+Create a new instance of the instance class.  C<@imports> should match the
+imports specified by C<$module>.  You can use a few shortcuts when specifying
+imports:
+
+=over 4
+
+=item code reference
+
+For a function import, you can provide a plan Perl subroutine, since we can
+determine the function signature from the C<$module>.
+
+=item scalar reference
+
+For a memory import, a memory object will be created and the referred scalar
+will be set to it.
+
+=item C<undef>
+
+For a memory import, a memory object will be created.  You won't be able to
+access it from Perl space, but at least it won't die.
+
+=back
 
 =head1 METHODS
 
