@@ -8,7 +8,7 @@ use Wasm::Wasmtime::Store;
 use Wasm::Wasmtime::MemoryType;
 
 # ABSTRACT: Wasmtime memory class
-our $VERSION = '0.02'; # VERSION
+our $VERSION = '0.03'; # VERSION
 
 
 $ffi_prefix = 'wasm_memory_';
@@ -66,6 +66,15 @@ $ffi->attach( grow => ['wasm_memory_t', 'uint32'] => 'bool' => sub {
   $xsub->($self->{ptr}, $delta);
 });
 
+
+# actually returns a wasm_extern_t, but recursion
+$ffi->attach( as_extern => ['wasm_memory_t'] => 'opaque' => sub {
+  my($xsub, $self) = @_;
+  require Wasm::Wasmtime::Extern;
+  my $ptr = $xsub->($self->{ptr});
+  Wasm::Wasmtime::Extern->new($ptr, $self->{owner} || $self);
+});
+
 $ffi->attach( [ delete => "DESTROY" ] => ['wasm_memory_t'] => sub {
   my($xsub, $self) = @_;
   if(defined $self->{ptr} && !defined $self->{owner})
@@ -88,7 +97,7 @@ Wasm::Wasmtime::Memory - Wasmtime memory class
 
 =head1 VERSION
 
-version 0.02
+version 0.03
 
 =head1 SYNOPSIS
 
@@ -165,6 +174,12 @@ Returns the current size of the memory in pages.
 
 Tries to increase the page size by the given C<$delta>.  Returns true on success, false otherwise.
 
+=head2 as_extern
+
+ my $extern = $memory->as_extern;
+
+Returns the L<Wasm::Wasmtime::Extern> for this memory object.
+
 =head1 AUTHOR
 
 Graham Ollis <plicease@cpan.org>
