@@ -4,11 +4,12 @@ use strict;
 use warnings;
 use Wasm::Wasmtime::FFI;
 use Wasm::Wasmtime::Store;
+use Wasm::Wasmtime::ImportType;
 use Wasm::Wasmtime::ExportType;
 use Carp ();
 
 # ABSTRACT: Wasmtime module class
-our $VERSION = '0.02'; # VERSION
+our $VERSION = '0.03'; # VERSION
 
 
 $ffi_prefix = 'wasm_module_';
@@ -120,9 +121,12 @@ $ffi->attach( exports => [ 'wasm_module_t', 'wasm_exporttype_vec_t*' ] => sub {
   $exports->to_list;
 });
 
-$ffi->attach( [ 'delete' => 'DESTROY' ] => ['wasm_module_t'] => sub {
+
+$ffi->attach( imports => [ 'wasm_module_t', 'wasm_importtype_vec_t*' ] => sub {
   my($xsub, $self) = @_;
-  $xsub->($self->{ptr}) if $self->{ptr};
+  my $imports = Wasm::Wasmtime::ImportTypeVec->new;
+  $xsub->($self->{ptr}, $imports);
+  $imports->to_list;
 });
 
 
@@ -144,6 +148,11 @@ sub get_export
   $self->{exports}->{$name};
 }
 
+$ffi->attach( [ 'delete' => 'DESTROY' ] => ['wasm_module_t'] => sub {
+  my($xsub, $self) = @_;
+  $xsub->($self->{ptr}) if $self->{ptr};
+});
+
 1;
 
 __END__
@@ -158,7 +167,7 @@ Wasm::Wasmtime::Module - Wasmtime module class
 
 =head1 VERSION
 
-version 0.02
+version 0.03
 
 =head1 SYNOPSIS
 
@@ -239,6 +248,12 @@ a useful diagnostic for why it was invalid.
 
 Returns a list of L<Wasm::Wasmtime::ExportType> objects for the objects exported by the WebAssembly module.
 
+=head2 imports
+
+ my @importtypes = $module->imports;
+
+Returns a list of L<Wasm::Wasmtime::ImportType> objects for the objects imported by the WebAssembly module.
+
 =head2 store
 
  my $store = $module->store;
